import os

import numpy as np

from fob.serialize import NumpyOutput, PickleOutput, RunOutput, StringOutput, TemporaryOutputPath


def test_numpyoutput(tmp_path):
    data = np.array([1, 2, 3])
    data2 = NumpyOutput.serialize(data, tmp_path)
    assert all(data == data2)
    assert all(NumpyOutput.deserialize(tmp_path) == data)


def test_runoutput(tmp_path):
    from trecrun import TRECRun

    data = TRECRun({1: {"d1": 1, "d2": 1.5}})
    data2 = RunOutput.serialize(data, tmp_path)
    assert data == data2
    assert RunOutput.deserialize(tmp_path) == data


def test_pickleoutput(tmp_path):
    data = {1: 2, 3: [4, 5, 6]}
    data2 = PickleOutput.serialize(data, tmp_path)
    assert data == data2
    assert PickleOutput.deserialize(tmp_path) == data


def test_stringoutput(tmp_path):
    data = "ahem"
    data2 = StringOutput.serialize(data, tmp_path)
    assert data == data2
    assert StringOutput.deserialize(tmp_path) == data


def test_temporary_output_path_unique(tmp_path):
    tmp_path_context = lambda: TemporaryOutputPath(tmp_path)
    for _ in range(100):
        with tmp_path_context() as path:
            assert not os.path.exists(path)
            os.makedirs(path, exist_ok=False)
